GradStore

Struct GradStore 

pub struct GradStore<T: FloatDType>(/* private fields */);

Implementations

Source

impl<T: FloatDType> GradStore<T>

pub fn new() -> Self

Create a new gradient store

pub fn get(&self, tensor: &Tensor<T>) -> Option<&Tensor<T>>

Get the gradient tensor associated with the given tensor

pub fn get_by_index(&self, index: usize) -> Option<&Tensor<T>>

pub fn remove(&mut self, tensor: &Tensor<T>) -> Option<Tensor<T>>

Remove the gradient tensor associated with the given tensor, returning it if it exists

pub fn insert( &mut self, tensor: &Tensor<T>, grad: Tensor<T>, ) -> Option<Tensor<T>>

Insert a gradient tensor associated with the given tensor, returning the previous gradient tensor if it existed

pub fn or_insert(&mut self, tensor: &Tensor<T>) -> Result<&mut Tensor<T>>

Get the gradient tensor associated with the given tensor, or, if it does not exist, insert a tensor of zeroes, with the same shape and type as the given tensors and return it

pub fn get_ids(&self) -> impl Iterator<Item = &TensorId>

Get the tensor ids of the stored gradient tensors

pub fn tensors(&self) -> impl Iterator<Item = &Tensor<T>>

pub fn iter(&self) -> Iter<'_, TensorId, Tensor<T>>

pub fn len(&self) -> usize